// On-Line Discovery of Dense Areas in Spatio-temporal Databases
// Copyright (C) 2003 Marios Hadjieleftheriou
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact information:
//  Mailing address:
//    Marios Hadjieleftheriou
//    University of California, Riverside
//    Department of Computer Science
//    Surge Building, Room 310
//    Riverside, CA 92521
//
//  Email:
//    marioh@cs.ucr.edu

/*
	Extracts useful statistics by comparing exact with approximate result files.
*/

#include "global.h"

class Entry
{
	public:
		int x, y, f;
		Entry(int a, int b, int c) : x(a), y(b), f(c) {}
};

int main(int argc, char** argv)
{
	if (argc != 3)
	{
		cerr << "Usage: " << argv[0] << " exhaustive_file estimate_file." << endl;
		return -1;
	}

	FILE* pFileEx = fopen(argv[1], "r");
	if (pFileEx == NULL)
	{
		cerr << "Cannot open " << argv[1] << "." << endl;
		return -1;
	}

	FILE* pFileEs = fopen(argv[2], "r");
	if (pFileEs == NULL)
	{
		cerr << "Cannot open " << argv[2] << "." << endl;
		return -1;
	}

	int datasetSize1, datasetSize2;
	int gridsize1, gridsize2;
	int density1, density2;
	int horizon1, horizon2;
	int totalTime1, totalTime2;
	int queryTime1, queryTime2;
	int updateTime1, updateTime2;
	int size1, size2;

	vector<Entry> ex[TMAX - TMIN][QUERIES_PER_TIME_INSTANT];
	vector<Entry> es[TMAX - TMIN][QUERIES_PER_TIME_INSTANT];
	int t, q, x, y, f, qt;

	char buf[1000];
	char test[10];
	test[5] = 0;

	while(!feof(pFileEx))
	{
		fgets(buf, 1000, pFileEx);
		memcpy(test, buf, 5);

		if (strcmp(test, "Info:") == 0)
		{
			sscanf(buf, "%s %d %d %d %d %d %d %d %d\n", test, &datasetSize1, &gridsize1, &density1, &horizon1, &totalTime1, &queryTime1, &updateTime1, &size1);
		}
		else
		{
			sscanf(buf, "%d %d %d %d %d\n", &t, &q, &x, &y, &f);
			Entry e = Entry(x, y, f);

			ex[t][q].push_back(e);
		}
	}

	while(!feof(pFileEs))
	{
		fgets(buf, 1000, pFileEs);
		memcpy(test, buf, 5);

		if (strcmp(test, "Info:") == 0)
		{
			sscanf(buf, "%s %d %d %d %d %d %d %d %d\n", test, &datasetSize2, &gridsize2, &density2, &horizon2, &totalTime2, &queryTime2, &updateTime2, &size2);

			if (datasetSize1 != datasetSize2 || horizon1 != horizon2 || density1 != density2 || gridsize1 != gridsize2)
			{
				cerr << "Files are incompatible. Cannot compare." << endl;
				return -1;
			}
		}
		else
		{
			sscanf(buf, "%d %d %d %d %d %d\n", &t, &q, &x, &y, &f, &qt);
			Entry e = Entry(x, y, f);

			es[t][q].push_back(e);
		}
	}

	fclose(pFileEs);
	fclose(pFileEx);

	// check again in case info line is missing.
	if (datasetSize1 != datasetSize2 || horizon1 != horizon2 || density1 != density2 || gridsize1 != gridsize2)
	{
		cerr << "Files are incompatible. Cannot compare." << endl;
		return -1;
	}

	unsigned long cFalsePositives = 0;
	unsigned long cMissed = 0;
	unsigned long cMatched = 0;
	double freqErr = 0.0;
	unsigned long cFreqErrCounter = 0;
	unsigned long missedPerTimeInstant[TMAX - TMIN];
	unsigned long exactPerTimeInstant[TMAX - TMIN];
	unsigned long estimatePerTimeInstant[TMAX - TMIN];
	double freqErrPerTimeInstant[TMAX - TMIN];
	unsigned long missedPerQuery[(TMAX - TMIN) * QUERIES_PER_TIME_INSTANT];
	unsigned long exactPerQuery[(TMAX - TMIN) * QUERIES_PER_TIME_INSTANT];
	unsigned long estimatePerQuery[(TMAX - TMIN) * QUERIES_PER_TIME_INSTANT];
	double freqErrPerQuery[(TMAX - TMIN) * QUERIES_PER_TIME_INSTANT];

	for (int cIndex = 0; cIndex < TMAX - TMIN; cIndex++)
	{
		missedPerTimeInstant[cIndex] = 0;
		exactPerTimeInstant[cIndex] = 0;
		estimatePerTimeInstant[cIndex] = 0;
		freqErrPerTimeInstant[cIndex] = 0.0;
	}

	/*
		for (int cIndex = 0; cIndex < (TMAX - TMIN) * QUERIES_PER_TIME_INSTANT; cIndex++)
		{
			missedPerQuery[cIndex] = 0;
			exactPerQuery[cIndex] = 0;
			estimateQuery[cIndex] = 0;
			freqErrPerQuery[cIndex] = 0.0;

		}
	*/

	int cQueryCount = 0;

	for (int T = 0; T < TMAX - TMIN; T++)
	{
		unsigned long cMatchedPerTimeInstant = 0;

		for (int cQuery = 0; cQuery < QUERIES_PER_TIME_INSTANT; cQuery++)
		{
			int cMatch = 0;
			int cNoMatch = 0;

			for (unsigned int cEx = 0; cEx < ex[T][cQuery].size(); cEx++)
			{
				bool found = false;

				for (unsigned int cEs = 0; cEs < es[T][cQuery].size(); cEs++)
				{
					if (ex[T][cQuery][cEx].x == es[T][cQuery][cEs].x && ex[T][cQuery][cEx].y == es[T][cQuery][cEs].y)
					{
						cMatchedPerTimeInstant++;
						cMatch++;
						found = true;
						double d = (double) abs((ex[T][cQuery][cEx].f - es[T][cQuery][cEs].f) / (double) ex[T][cQuery][cEx].f);
						freqErr += d;
						freqErrPerTimeInstant[T] += d;
						cFreqErrCounter++;
						break;
					}
				}

				if (! found)
				{
					cNoMatch++;
					cerr << "Missed: " << T << " " << cQuery << ", x: " << ex[T][cQuery][cEx].x << ", y: " << ex[T][cQuery][cEx].y << ", f: " << ex[T][cQuery][cEx].f << endl;
				}
			}

			cMissed += cNoMatch;
			cMatched += cMatch;
			cFalsePositives += (es[T][cQuery].size() - cMatch);
			missedPerTimeInstant[T] += cNoMatch;
			exactPerTimeInstant[T] += ex[T][cQuery].size();
			estimatePerTimeInstant[T] += es[T][cQuery].size();

			missedPerQuery[cQueryCount] = cNoMatch;
			exactPerQuery[cQueryCount] = ex[T][cQuery].size();
			estimatePerQuery[cQueryCount] = es[T][cQuery].size();
			cQueryCount++;
		}

		freqErrPerTimeInstant[T] /= (double) cMatchedPerTimeInstant;
	}

	cout << "Dataset size: " << datasetSize1 << ", Horizon: " << horizon1 << ", Density: " << density1 << endl;
	cout << "Update time: Exact:" << updateTime1 << ", Estimate: " << updateTime2 << endl;
	cout << "Query time: Exact:" << queryTime1 << ", Estimate: " << queryTime2 << endl;
	cout << "Total time: Exact:" << totalTime1 << ", Estimate: " << totalTime2 << endl;
	cout << "Size: Exact: " << size1 << ", Estimate: " << size2 << ", %: " << 100 * ((double) size2 / (double) size1) << endl;
	cout << "False Positives: " << cFalsePositives << endl;
	cout << "Missed: " << cMissed << endl;
	cout << "Matched: " << cMatched << endl;
	cout << "Average density error: " << freqErr / (double) cFreqErrCounter << endl;

	for (int cIndex = 0; cIndex < TMAX - TMIN; cIndex++)
	{
		if (exactPerTimeInstant[cIndex] > 0)
			cout << "T: " << cIndex << ", Missed: " << missedPerTimeInstant[cIndex] << " / " << exactPerTimeInstant[cIndex]
				<< ", Found: " << estimatePerTimeInstant[cIndex] << ", Error: " << freqErrPerTimeInstant[cIndex] << endl;
	}

	for (int cIndex = 0; cIndex < (TMAX - TMIN) * QUERIES_PER_TIME_INSTANT; cIndex++)
	{
		cout << "Query: " << cIndex << ", Missed: " << missedPerQuery[cIndex] << " / " << exactPerQuery[cIndex]
			<< " / " << estimatePerQuery[cIndex] << ", Error: " << freqErrPerQuery[cIndex] << endl;
	}

	return 0;
}
