// On-Line Discovery of Dense Areas in Spatio-temporal Databases
// Copyright (C) 2003 Marios Hadjieleftheriou
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact information:
//  Mailing address:
//    Marios Hadjieleftheriou
//    University of California, Riverside
//    Department of Computer Science
//    Surge Building, Room 310
//    Riverside, CA 92521
//
//  Email:
//    marioh@cs.ucr.edu

#include "global.h"

void initRandom()
{
	srand48(time(NULL));
}

long nextUniformLong(long low, long high)
{
	return (long) floor(((double) low + ((double) (high - low)) * drand48()));
}

int nextUniformInt(int low, int high)
{
	return (int) floor(((double) low + ((double) (high - low)) * drand48()));
}

double nextUniformDouble(double low, double high)
{
	return (low + ((high - low) * drand48()));
}

double nextUniformDouble()
{
	return drand48();
}

// mean 0.0, standard deviation 1.0
double nextGaussianDouble()
{
	static bool haveNextNextGaussian = false;
	static double nextNextGaussian;

	if (haveNextNextGaussian)
	{
		haveNextNextGaussian = false;
		return nextNextGaussian;
	}
	else
	{
		double v1, v2, s;

		do
		{
	 // between -1.0 and 1.0
			v1 = 2 * drand48() - 1;
	 // between -1.0 and 1.0
			v2 = 2 * drand48() - 1;
			s = v1 * v1 + v2 * v2;
		} while (s >= 1 || s == 0);

		double multiplier = std::sqrt(-2 * std::log(s)/s);
		nextNextGaussian = v2 * multiplier;
		haveNextNextGaussian = true;
		return v1 * multiplier;
	}
}

double nextZipfDouble(double low, double high, double p)
{
	double x;
	int i;
	double r, HsubV, sum;
	int V = 100;

	/* calculate the V-th harmonic number HsubV. WARNING: V>1 */
	HsubV = 0.0;
	for(i = 1; i <= V; i++) HsubV += 1.0 / pow((double) i, p);

	r = nextUniformDouble() * HsubV;
	sum = 1.0; i=1;
	while(sum < r)
	{
		i++;
		sum += 1.0/ pow((double)i, p);
	}

	/* i follows Zipf distribution and lies between 1 and V */
	/* x lies between 0. and 1. and then between low and high */
	x = ((double) i - 1.0) / ((double) V - 1.0);
	x = (high - low) * x + low;

	return x;
}

double nextGaussianDouble(double m, double std)
{
	return m + (std * nextGaussianDouble());
}

bool flipCoin()
{
	if (nextUniformDouble() >= 0.5) return true;
	return false;
}

ostream& operator<<(ostream& os, const MovingObject& o)
{
	os << o.m_id << " " << o.m_st << " " << o.m_sx << " " << o.m_sy << " " << o.m_vx << " " << o.m_vy;

	return os;
}
