// On-Line Discovery of Dense Areas in Spatio-temporal Databases
// Copyright (C) 2003 Marios Hadjieleftheriou
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact information:
//  Mailing address:
//    Marios Hadjieleftheriou
//    University of California, Riverside
//    Department of Computer Science
//    Surge Building, Room 310
//    Riverside, CA 92521
//
//  Email:
//    marioh@cs.ucr.edu

#ifndef _global_h
#define _global_h

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <errno.h>
#include <math.h>

#include <string>
#include <iostream>
#include <cmath>
#include <sstream>
#include <vector>
#include <map>
#include <set>
#include <stack>
#include <queue>

using namespace std;

#define DELETE 0
#define INSERT 1
#define QUERY 2

#define GAUSSIAN 0
#define UNIFORM 1
#define ZIPF 2

#define SNAPSHOT 0
#define RANGE 1
#define TOPK 2

#define QUERIES_PER_TIME_INSTANT 2

#define TMIN 0
#define TMAX 100

#define XMIN 0.0
#define XMAX 100.0
#define YMIN 0.0
#define YMAX 100.0

class Query
{
	public:
		double m_x1, m_x2, m_y1, m_y2;
		int m_t1, m_t2;

		Query(double x1, double x2, double y1, double y2, int t1, int t2)
			:  m_x1(x1), m_x2(x2), m_y1(y1), m_y2(y2), m_t1(t1), m_t2(t2)
		{
		}
};

class Bucket
{
	public:
		double m_low[2];
		double m_high[2];
		unsigned int m_count;

		bool contains(double x, double y)
		{
			if (m_low[0] <= x && x <= m_high[0] && m_low[1] <= y && y <= m_high[1]) return true;
			return false;
		}
};

class MovingObject
{
	public:
		int m_id;
		int m_st;
		double m_sx;
		double m_sy;
		double m_vx;
		double m_vy;
		int m_kt;

		MovingObject() {}

		MovingObject(int id, double x, double y, double vx, double vy, int t)
		{
			this->m_id = id;
			this->m_sx = x;
			this->m_sy = y;
			this->m_vx = vx;
			this->m_vy = vy;
			this->m_st = t;
		}

		double getBX()
		{
			return -(getAX() / m_vx);
		}

		double getBY()
		{
			return -(getAY() / m_vy);
		}

		double getNX()
		{
			return 1.0 / m_vx;
		}

		double getNY()
		{
			return 1.0 / m_vy;
		}

		double getAX()
		{
			return m_sx - m_vx * (double) m_st;
		}

		double getAY()
		{
			return m_sy - m_vy * (double) m_st;
		}

		double getVX()
		{
			return m_vx;
		}

		double getVY()
		{
			return m_vy;
		}

		double getX(int t)
		{
			return m_sx + ((double) (t - m_st)) * m_vx;
		}

		double getY(int t)
		{
			return m_sy + ((double) (t - m_st)) * m_vy;
		}
};

extern void initRandom();
extern long nextUniformLong(long, long);
extern int nextUniformInt(int, int);
extern double nextUniformDouble();
extern double nextUniformDouble(double, double);
extern double nextZipfDouble(double, double, double);
extern double nextGaussianDouble();
extern double nextGaussianDouble(double, double);
extern bool flipCoin();

extern ostream& operator<<(ostream& os, const MovingObject& o);
#endif
